Index: src/main/java/jpen/provider/xinput/XiDevice.java
===================================================================
--- src/main/java/jpen/provider/xinput/XiDevice.java	(revision 244)
+++ src/main/java/jpen/provider/xinput/XiDevice.java	(working copy)
@@ -155,6 +155,14 @@
 	}
 
 	private static native int getLastEventButton(int cellIndex);
+	
+	public boolean getLastEventProximity() {
+		synchronized(xiBus){
+			return getLastEventProximity(cellIndex);
+		}
+	}
+	
+	private static native boolean getLastEventProximity(int cellIndex);
 
 	public void refreshLevelRanges(){
 		synchronized(xiBus){
@@ -196,6 +204,8 @@
 			sb.append(getLastEventType());
 			sb.append(", lastEventButton=");
 			sb.append(getLastEventButton());
+			sb.append(", lastEventProximity=");
+			sb.append(getLastEventProximity());
 			sb.append("}");
 			return sb.toString();
 		}
Index: src/main/c/linux/Device.c
===================================================================
--- src/main/c/linux/Device.c	(revision 244)
+++ src/main/c/linux/Device.c	(working copy)
@@ -29,6 +29,8 @@
 			{ButtonClass,_deviceButtonPress},
 			{ButtonClass,_deviceButtonRelease},
 			{ValuatorClass, _deviceMotionNotify},
+			{ProximityClass, _proximityIn},
+			{ProximityClass, _proximityOut}
 		};
 
 void Device_setIsListening(SDevice *pDevice, int isListening) {
@@ -211,6 +213,18 @@
 					Device_refreshValuatorValues(pDevice, pEvent->first_axis, pEvent->axes_count, pEvent->axis_data);
 				}
 				break;
+			case E_EventType_ProximityIn:
+			case E_EventType_ProximityOut:
+				{
+					XProximityNotifyEvent *pEvent = (XProximityNotifyEvent *)&pDevice->lastEvent;
+					pDevice->lastEventTime=pEvent->time;
+					pDevice->lastEventProximity=(i==E_EventType_ProximityIn);
+					pDevice->lastEventButton=pEvent->device_state;
+					Device_refreshValuatorValues(pDevice, pEvent->first_axis, pEvent->axes_count, pEvent->axis_data);
+				}
+				break;
 			default:
 				printf("assertion error: Device_waitNextEvent: unexpected unhandled event\n");
 				return false;
Index: src/main/c/linux/Device.h
===================================================================
--- src/main/c/linux/Device.h	(revision 244)
+++ src/main/c/linux/Device.h	(working copy)
@@ -26,6 +26,8 @@
 	E_EventType_ButtonPress,
 	E_EventType_ButtonRelease,
 	E_EventType_MotionNotify,
+	E_EventType_ProximityIn,
+	E_EventType_ProximityOut,
 	E_EventType_size
 };
 /* Warning: this enumeration is analogous to the PLevel.Type enumeration. */
@@ -56,6 +58,7 @@
 	int lastEventType;
 	Time lastEventTime;
 	int lastEventButton;
+	int lastEventProximity;
 };
 m_declareRow(Device);
 extern int Device_init(SDevice *pDevice, SBus *pBus, int deviceIndex);
Index: src/main/c/linux/jpen_provider_xinput_XiDevice.c
===================================================================
--- src/main/c/linux/jpen_provider_xinput_XiDevice.c	(revision 244)
+++ src/main/c/linux/jpen_provider_xinput_XiDevice.c	(working copy)
@@ -150,6 +150,16 @@
 }
 
 /*
+ * Class:     jpen_provider_xinput_Device
+ * Method:    getLastEventProximity
+ * Signature: (I)I
+ */
+JNIEXPORT jboolean JNICALL Java_jpen_provider_xinput_XiDevice_getLastEventProximity
+(JNIEnv *pEnv, jclass class, jint cellIndex) {
+	return Device_getP(cellIndex)->lastEventProximity? JNI_TRUE:JNI_FALSE;
+}
+
+/*
  * Class:     jpen_provider_xinput_XiDevice
  * Method:    refreshLevelRanges
  * Signature: (I)V

